import com.j256.ormlite.dao.Dao;
import com.j256.ormlite.dao.DaoManager;
import com.j256.ormlite.jdbc.JdbcConnectionSource;
import com.j256.ormlite.support.ConnectionSource;

import java.sql.SQLException;
import java.util.InputMismatchException;
import java.util.Scanner;

public class AccountApp {

    // we are using a MySQl database
    private final static String DATABASE_URL = "jdbc:mysql://localhost:3306/orm_lab?useUnicode=true";

    private Dao<Account, Integer> accountDao;

    private boolean control = true;

    public AccountApp(){
        try {
            ConnectionSource connectionSource = new JdbcConnectionSource(DATABASE_URL, "root", "");
            setupDao(connectionSource);
        } catch (SQLException e) {
            e.printStackTrace();
        }catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) throws Exception {
        // turn our static method into an instance of Main
        new AccountApp().doMain(args);
    }

    private void doMain(String[] args) throws Exception {
        ConnectionSource connectionSource = null;

        try {
            processData();

            System.out.println("\n\nIt seems to have worked\n\n");
        } finally {
            // destroy the data source which should close underlying connections
            if (connectionSource != null) {
                connectionSource.close();
            }
        }
    }

    /**
     * Read and write some example data.
     */
    private void processData() throws Exception {
        while (control){
            String command = getInput("What would you like to do: 'c' , 'r', 'u', 'd' \n" +
                                      "Type anything else qo quit.");

            if (command.equalsIgnoreCase("c")){
                String name = getInput("Enter name for account: \n");
                String pass = getInput("Enter pass for account: \n");

                createAcc(createAccObj(name, pass));

            } else if (command.equalsIgnoreCase("r")){
                System.out.println(readAcc(readAccId()));
            } else if (command.equalsIgnoreCase("u")){
                updateAcc(readAccId(), getInput("Enter new password: "));
            } else if (command.equalsIgnoreCase("d")){
                deleteAcc(readAccId());
            } else {
                System.out.println("Thanks for coming.");
                break;
            }
        }

    }



    public Account createAccObj(String name, String password){
        Account acc = new Account (name, password);
        return acc;
    }

    public void createAcc(Account acc) throws SQLException {
        accountDao.create(acc);
        //accountDao.update(acc);
    }

    public int readAccId(){
        int id = Integer.parseInt(getInput("Type ID of your user: \n"));
        return id;
    }

    public String readAcc(int id) throws SQLException {
        if (accountDao.queryForId(id) == null){
            return null;
        }
        String readUser = ("ID: " + accountDao.queryForId(id).getId() + " ") +
                "Name: " + accountDao.queryForId(id).getName() + " " +
                "Pass: " + accountDao.queryForId(id).getPassword();
        return readUser;
    }

    public void updateAcc(int id, String password) throws SQLException{
        Account acc = accountDao.queryForId(id);
        acc.setPassword(password);
        accountDao.update(acc);
    }

    public void deleteAcc(int id) throws SQLException {
        Account acc = accountDao.queryForId(id);
        accountDao.deleteById(id);
        System.out.println(acc.getName() + " has been deleted.");
    }


    private String getInput(String message){
        Scanner input = new Scanner(System.in);
        System.out.println(message);
        String newInput = input.nextLine();
        return newInput;
    }


    /**
     * Setup our  DAOs
     */
    private void setupDao(ConnectionSource connectionSource) throws Exception {

        accountDao = DaoManager.createDao(connectionSource, Account.class);

    }
}
